/*
IMAP.h
*/

#ifndef __IMAP_H
#define __IMAP_H

#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <signal.h>

#include "c-client.h"

#include "Language.h"
#include "Utils.h"
#include "Config.h"
#include "XString.h"
#include "XVector.h"
#include "Codecs.h"

typedef XVector<XString> StringList;

extern "C" long doTCPTimeout (long elapsed, long sincelast);
extern "C" void mm_flags (MAILSTREAM *stream,xulong number);
extern "C" void mm_status (MAILSTREAM *stream,char *mailbox,MAILSTATUS *status);
extern "C" void mm_list (MAILSTREAM *stream,int delim,char *name,long attrib);
extern "C" void mm_notify (MAILSTREAM *stream,char *string,long errflg);
extern "C" void mm_log (char *string,long errflg);
extern "C" void mm_dlog (char *string);
extern "C" void mm_login (NETMBX *mb,char *user,char *pwd,long trial);
extern "C" long mm_diskerror (MAILSTREAM *stream,long errcode,long serious);
extern "C" void mm_critical (MAILSTREAM *stream);
extern "C" void mm_nocritical (MAILSTREAM *stream);
extern "C" void mm_searched (MAILSTREAM *stream,xulong number);
extern "C" void mm_exists (MAILSTREAM *stream,xulong number);
extern "C" void mm_expunged (MAILSTREAM *stream,xulong number);
extern "C" void mm_lsub (MAILSTREAM *stream,int delim,char *name,long attrib);
extern "C" void mm_fatal (char *string);

extern "C" void mail_fetchfrom_mod (char *s,MAILSTREAM *stream,unsigned long msgno, long length, int option);
extern "C" void mail_fetchto_mod (char *s,MAILSTREAM *stream,unsigned long msgno, long length, int option);

class IMAP
  {
  private:
    bool TheDEBUG, TheERROR, SortReversed;
    MAILSTREAM *imapstream;
    TFileName imaphost, username, password, mailbox, fullmailbox;
    StringList *SLMailboxes, *SLSearch;
    ULongList *ILSort;
    XString XSearch;
    TBuffer MailboxPrefix, PathToRemoteFolder;
    TSBuffer ERRORSTRING;
    int imapport, timeout, maxlogintrials, msgsmarkeddeleted, SortType, SortType2;
    char delimiter;
    long MAXMSGSFORPAGE, MAILBOXSIZE, TOTALMSGS;
    void display_body (BODY *body, char *pfx, long i, StringList *SLAttachs);
    char *getFullMailbox (char *amailbox);
    char *getBriefMailbox (char *amailbox);
    void doMailboxSize (void);
  public:
    IMAP ();
    virtual ~IMAP ();
    bool isDebug (void);
    void setMAXMSGSFORPAGE (long maxmsgs);
    MAILSTREAM *getStream (void);
    xulong getNumMsgFromPosList (xulong pos);
    xulong getPosListFromNumMsg (xulong nummsg);
    bool isLastMsgFromList (xulong nummsg);
    char *getUsername (void);
    char *getPassword (void);
    bool OpenMailbox (const char *aimaphost, int aimapport, char *amailbox, char *ausername, char *apassword);
    bool Reconnect (void);
    bool ChangeMailbox (char *newmailbox);
    void addMailbox (char *name);
    void CloseMailbox (void);
    void Expunge (void);    
    long getTotalMsgs (void);
    void setPathToRemoteFolder (char *aPathToRemoteFolder);
    char *getPathToRemoteFolder (void);    
    void setMailboxPrefix (char *aMailboxPrefix);
    char *getMailboxPrefix (void);
    bool getError (void);    
    void setError (bool value);
    char *getErrorString (void);
    void setErrorString (char *sterr);
    long getRecentMsgs (void);
    char *getMailboxName (void);
    char *getFullMailboxName (void);
    void setTimeout (int atimeout);
    void disableRSH (void);
    void hideDotFiles (void);
    void setDelimiter (char adelim);
    char getDelimiter (void);
    void setMaxLoginTrials (int amaxlogintrials);
    bool Ping (void);
    char *getAttIdFromCID (xulong nummsg, char *cid);
    int getMsgsMarkedDeleted (void);
    long getMailboxSize (void);
    void getHeaderList (xulong abeginmsg, StringList *SL, char *user, char *maildomain, char *tolabel);
    char *getAttHeaderPC (xulong nummsg, char *attid);    
    void getAttHeaderSL (xulong nummsg, char *attid, StringList *SL);    
    char *dumpFullMsg (xulong nummsg);
    void dumpMsg (xulong nummsg, StringList *SL);
    void dumpFullHeader (xulong nummsg, StringList *SL);
    void dumpFullBody (xulong nummsg, StringList *SL);
    char *getTo (xulong nummsg);
    char *getCC (xulong nummsg);
    char *getReplyTo (xulong nummsg);
    char *getFrom (xulong nummsg);
    char *getContentType (xulong nummsg);    
    char *getSubject (xulong nummsg);
    char *getDate (xulong nummsg);
    char *getFlags (xulong nummsg);
    xulong getSize (xulong nummsg);
    char *getFormattedSize (xulong nummsg);
    void getBodyStructure (xulong nummsg, StringList *SLAttach);
    char *getAttBody (xulong nummsg, char *attid, StringList *SLAttachInfo, xulong *len, bool decodificar);    
    xulong getAttSize (xulong nummsg, char *attid, StringList *SLAttachInfo);    
    bool getAttBodyToFile (xulong nummsg, char *attid, StringList *SLAttachInfo, xulong *len, char *fname, bool decodificar);    
    void getAttachInfo (BODY *body, char *oripfx, char *pfx, long i, StringList *SLAttach, StringList *SLFullHeader);
    int setFlags (char *flags, char *pcmessages);
    int clearFlags (char *flags, char *pcmessages);    
    void getMailboxes (StringList *SL, bool refresh);
    bool createMailbox (char *mailboxname);
    bool deleteMailbox (char *mailboxname);
    bool renameMailbox (char *oldmailboxname, char *newmailboxname);
    int copyMessages (char *mailboxname, char *pcmessages);
    int moveMessages (char *mailboxname, char *pcmessages);
    bool appendMsg (char *mailbox, STRING *msg);
    int searchIMAP2 (char *range, int imaptoken, char *whatsearch, char *from, char *fecha);
    void addResultToSearch (xulong msgnum);
    bool delMsgInSearchList (xulong msgnum);
    void addMsgInSearchList (xulong msgnum);
    bool IsSearched (xulong nummsg);
    xulong getNumFirstMsgInSearch (void);
    void setSortType (int aSortType);
    int Sort (void);
    void getMsgIndex (ULongList *IL);
    void sortMailboxes (StringList *SL);
    void PathTool (const char *mailbox, int *numseps, int *poslastsep);
    char *expandMailboxName (const char *mailbox);
  };
  
#endif  


